package net.sf.xanswer.storage.map;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URI;
import java.util.Random;

public abstract class PersistentFactoryImplBase implements PersistentFactory {
    protected abstract String getNamePrefix();
    protected abstract PersistentMap instantiateDbApi();
    protected abstract boolean isDirectoryDatabase();

    private File myDirectory;
    private Random myRandom;

    public void setDirectory(URI directory) {
        myRandom = new Random();
    	File newDirectory = new File(directory);
    	if (!newDirectory.isDirectory()) throw new IllegalArgumentException("URI is not a directory");
		myDirectory = newDirectory;
    }

    public PersistentMap newDatabase() throws PersistentException {
        PersistentMap result = null;
   	    // - may be 233 is better :) ?
        // - 333 is MUCH better :)))
   		String randomName = getNamePrefix()+String.valueOf(myRandom.nextInt(333));
   		try {
   	    	result = newDatabase(randomName);
   		} catch (PersistentException e) {
   		    if (e.getMessage().indexOf(randomName) >= 0) {
   		        result = newDatabase();
   		    } else {
   		        throw e;
   		    }
   		}
    	return result;
    }

    //TODO: newDatabase(String databaseName)
    public PersistentMap newDatabase(String databaseName) throws PersistentException {
        return newDatabase(databaseName, DEFAULT_PAGE_SIZE, DEFAULT_BUFFERS_COUNT);
    }

    // public only temporarily, for Performance tests only?
    public PersistentMap newDatabase(String databaseName, int pageSize, int buffersCount) throws PersistentException {
    	if (myDirectory == null) throw new IllegalStateException("I don't know where to place db files!");
		File newFile = getFileFromName(databaseName);
		if (newFile.exists()) {
		    throw new PersistentException(new FileNotFoundException(databaseName+": "+newFile.getAbsolutePath()+" already exists"));
		}
    	if (isDirectoryDatabase()) {
    	    newFile.mkdir();
    	}
    	PersistentMap result = instantiateDbApi();
    	result.create(newFile.getAbsolutePath(), pageSize, buffersCount);
    	return result;
    }
    
    //TODO: use relative path?
    
    // now we use databaseName (it is a relative file name without extension or relative directory name as is)
    // - is it OK? :-\
    public PersistentMap getDatabase(String databaseName) throws PersistentException {
    	if (myDirectory == null) throw new IllegalStateException("I don't know where to find db files!");
        File locationFile = getFileFromName(databaseName);
        if (!locationFile.exists()) {
            throw new PersistentException(new IOException("Database "+databaseName+" does not exist in my directory: "+myDirectory));
        }
        if (isDirectoryDatabase() ^ locationFile.isDirectory()) {
            throw new PersistentException(new IOException("Not appropriate directory or file kind for this database"));
        }
    	PersistentMap result = instantiateDbApi();
    	result.create(locationFile.getAbsolutePath());
        return result;
    }
    
    public String getInstanceName() {
        return getNamePrefix();
    }
    
    private File getFileFromName(String databaseName) {
        String extension = isDirectoryDatabase() ? "" : ".db";
        return new File(myDirectory, databaseName+extension);
    }

}
