package org.exist;

import java.io.File;
import java.io.UnsupportedEncodingException;

import net.sf.xanswer.storage.map.PersistentMapImplBase;
import net.sf.xanswer.storage.map.PersistentException;
import net.sf.xanswer.storage.map.PersistentFactory;

import org.dbxml.core.DBException;
import org.dbxml.core.data.Value;
import org.dbxml.core.filer.BTreeException;
import org.dbxml.core.filer.Paged;
import org.dbxml.core.filer.ValuesIterator;
import org.exist.storage.store.BFileUllman;

/**
 * This class provides the modification of eXist implementaion of PersistentMap.
 */
public class PersistentMapImplUllman extends PersistentMapImplBase /*implements PersistentMap */{
	private BFileUllman myBFile;
	
	PersistentMapImplUllman(PersistentFactory factory) {
		super(factory);
	}
	
    protected String getInstanceName() {
    	return "Ullman";
    }

    // TODO: provide a balancing mechanism for the b-tree algorithm and test it.
    protected void createDatabase(File location, int pageSize, int buffersCount) throws PersistentException {
		Paged.setPageSize(pageSize); // in bytes
		// in Cacheable items (Pages)(Nodes) - 256 by default in both caches:
	    myBFile = new BFileUllman(location, buffersCount/*BTree Cache*/, 256/*BFile DataCache - 256 by default*/);
	    if (!myBFile.exists()) {
		    try {
               myBFile.create();
           } catch (DBException e) {
               throw new PersistentException(e);
           } 
	    }
    }

    public void openDatabase() throws PersistentException {
        try {
        	myBFile.open();
        } catch (DBException e) {
            throw new PersistentException(e);
        }
    }

    public void closeDatabase() throws PersistentException {
        try {
        	myBFile.flush();
        	myBFile.close();
        } catch (DBException e) {
            throw new PersistentException(e);
        }
    }

	protected void putBytes(String key, byte[] value) throws PersistentException {
			myBFile.put(new Value(key), value, true);
//			System.err.println("put key: " + key + " value: " + value);
	}

	protected byte[] getBytes(String key) throws PersistentException {
		byte[] result = new byte[0];
		Value value = myBFile.get(new Value(key));
		if (value != null) {
			result = value.getData(); 		    
		}
//		if ((result != null) && (result.length != 0)) System.err.println("get key: " + key + " value: " + result);
	    return result;
	}

	protected void removeBytes(String key) throws PersistentException {
		myBFile.remove(new Value(key));
	}

    protected StringIterator getIterator() throws PersistentException {
        return new StringIteratorImplUllman(myBFile.iterator());
    }
	
    protected long getNumberOfRecords() throws PersistentException {
        int result = 0;
        StringIterator it = getIterator();
        while (it.hasNext()) {
            result = ++result;
            it.next();
        }
        return result;
    }
    
    private class StringIteratorImplUllman implements StringIterator {
        ValuesIterator myIterator;
        
        protected StringIteratorImplUllman(ValuesIterator iterator) {
            myIterator = iterator;
        }

        public boolean hasNext() throws PersistentException {
            boolean result = false;
            try {
                result = myIterator.hasNext();
            } catch (BTreeException e) {
    			throw new PersistentException(e);
            }
            return result;
        }

        public String next() throws PersistentException {
            String result = "";
            try {
                Value nextValue = myIterator.next();
                if (nextValue != null) {
    				result = new String(nextValue.getData(), "UTF-8");                    
                }
            } catch (BTreeException e) {
    			throw new PersistentException(e);
            } catch (UnsupportedEncodingException e) {
    			throw new PersistentException(e);
            }            
            return result;
        }

    }

}
